import numpy as np
import matplotlib.pyplot as plt
import math


def lcm(a, b):
    return abs(a*b) // math.gcd(a, b)


def spirograph(r1, r2, r3, dtheta=0.05):
    # round number radii are assumed
    r1_round = round(abs(r1))
    r2_round = round(abs(r2))
    r1_r2_lcm = lcm(r1_round, r2_round)
    turns = r1_r2_lcm/r2_round

    n = round(2*math.pi*turns/dtheta)
    theta = np.linspace(0, 2*math.pi*turns, n)
    v1 = np.zeros((n, 2))
    v2 = np.zeros((n, 2))

    roll = r2*theta
    theta_main = roll/r1

    v1[:, 0] = r1*np.cos(theta_main)
    v1[:, 1] = r1*np.sin(theta_main)
    v2[:, 0] = r3*np.cos(theta_main+theta)
    v2[:, 1] = r3*np.sin(theta_main+theta)

    xy = v1+v2

    return xy


def repeat_shape(shape, theta, scale, n):
    n_p = shape.shape[0]
    shape_tot = np.zeros((n, n_p, 2))

    R = scale*np.array([[math.cos(theta), math.sin(theta)],
                        [-math.sin(theta), math.cos(theta)]])
    for i in range(n):
        shape_tot[i, :, :] = shape
        shape = np.dot(shape, R)
    return shape_tot


def scale_move(xy, scale, xy_delta):
    xy_new = xy*scale + np.array([xy_delta])
    return xy_new


def fit_rectangle(shape, x1, y1, x2, y2):
    n_dim = len(shape.shape)
    min_all = np.min(shape, axis=tuple(range(n_dim-1)))
    max_all = np.max(shape, axis=tuple(range(n_dim-1)))
    x1_s, y1_s = min_all[:]
    x2_s, y2_s = max_all[:]

    w, h = x2-x1, y2-y1
    w_s, h_s = x2_s-x1_s, y2_s-y1_s
    ratio = w/h
    ratio_s = w_s/h_s
    center = np.array([x1*0.5+x2*0.5,
                       y1*0.5+y2*0.5])
    center_s = np.array([x1_s*0.5+x2_s*0.5,
                         y1_s*0.5+y2_s*0.5])

    use_x = ratio < ratio_s
    scale = w/w_s if use_x else h/h_s

    shape_new = (shape-center_s)*scale + center
    return shape_new


def plot_curves(shape, style="b"):
    n_dim = len(shape.shape)
    if n_dim == 2:
        shape = shape[np.newaxis, :, :]

    plt.figure()
    for i in range(len(shape)):
        xy = shape[i, :, :]
        plt.plot(xy[:, 0], xy[:, 1], style)
    plt.axis('equal')
    plt.show()
